#!/usr/bin/python

import ldap
import urllib
import string
import ldap.dn
import ldap.filter

xmlns = "http://asp.mexmat.net/xmlns/iml"

def nodeSkip(node):
	if node.name == "text": return True
	return False

def nodeSkipNs(node):
	if not node.ns(): return True 
	if node.ns().content != xmlns: True
	return nodeSkip(node)

class IMLFile:
	def __init__(self, tuple, xml = None):
		self.file = []
		self.lang = []
		if xml:
			return self.parse_xml(xml)
		(self.dn, self.attr) = tuple
		self.cn = self.attr["cn"][0]
		if self.attr.has_key("imlFileURL"):
			self.file = self.attr["imlFileURL"]
		if self.attr.has_key("imlFileLang"):
			self.lang = self.attr["imlFileLang"]
	def xml(self):
		file = string.join(map (lambda x: "<iml:url href='%s'/>" % x, self.file))
		lang = string.join(map (lambda x: "<iml:lang href='%s'/>" % x, self.lang))
		return """
			<iml:file id='%s' file='%s'>%s\n%s\n</iml:file>
		""" % (urllib.quote(self.dn), self.cn, file, lang)
	def parse_xml(self, xml):
		if xml.name != "file":
			raise RuntimeError("Failed to parse IML file: %s" % xml.name)
		self.dn = ""
		self.cn = ""
		node = xml.children
		while node:
			if nodeSkipNs(node):
				pass
			elif node.name == "url":
				def urlfold(x):
					if x.type == "attribute" and x.name == "href": 
						self.file.append(urllib.unquote(x.content))
				map(urlfold, list(node.properties))
			elif node.name == "lang":
				self.lang.append(urllib.unquote(node.content))
			node = node.next
		node = xml.properties
		while node:
			if nodeSkip(node):
				pass
			elif node.name == "id":
				self.dn = urllib.unquote(node.content)
			elif node.name == "file":
				self.cn = urllib.unquote(node.content)
			elif node.name == "size":
				self.size = int(node.content)
			node = node.next

class IMLFilm:
	def __init__(self, tuple, verbose = False, xml = None):
		self.file = []
		self.gn = []
		self.actors = []
		self.director = None
		self.year = None
		self.country = []
		self.genre = []
		if xml:
			verbose = True
			return self.parse_xml(xml)
		(self.dn, self.attr) = tuple
		self.cn = self.attr["cn"][0]
		self.verbose = verbose
		if self.attr.has_key("givenName"):
			self.gn = self.attr["givenName"]
		if self.attr.has_key("imlFilmActor"):
			self.actors = self.attr["imlFilmActor"]
		if self.attr.has_key("imlFilmDirector"):
			self.director = self.attr["imlFilmDirector"][0]
		if self.attr.has_key("imlFilmYear"):
			self.year = self.attr["imlFilmYear"][0]
		if self.attr.has_key("imlFilmGenre"):
			self.genre = self.attr["imlFilmGenre"]
		if self.attr.has_key("co"):
			self.country = self.attr["co"]
	def xml(self, mode = None):
		str = ""
		if not mode: 
			if self.verbose:
				mode = "long"
			else:
				mode = "short"
		if mode != "short":
			str = string.join(map (lambda x: x.xml(), self.file)) + "\n"
			str = str + string.join(map (lambda x: "<iml:title>" + x + "</iml:title>", self.gn)) + "\n"
			str = str + string.join(map (lambda x: "<iml:actor>" + x + "</iml:actor>", self.actors)) + "\n"
			str = str + string.join(map (lambda x: "<iml:country>" + x + "</iml:country>", self.country)) + "\n"
			str = str + string.join(map (lambda x: "<iml:genre>" + x + "</iml:genre>", self.genre)) + "\n"
			if self.director:
				str = str + "<iml:director>" + self.director + "</iml:director>" + "\n"
			if self.year:
				str = str + "<iml:year>" + self.year + "</iml:year>" + "\n"
		return """ <iml:film id='%s' mode='%s'> <iml:title main='main'>%s</iml:title>\n %s\n </iml:film>
			""" % (urllib.quote(self.dn), mode,self.cn, str)
	def files(self, tuple):
		self.file = self.file + map (lambda x: IMLFile(x), tuple)
	def parse_xml(self, xml):
		if xml.name != "film":
			raise RuntimeError("Failed to parse IML film: %s" % xml.name)
		self.dn = ""
		self.cn = ""
		node = xml.children
		self.verbose = True
		while node:
			if nodeSkipNs(node):
				pass
			elif node.name == "file":
				self.file.append(IMLFile([], xml = node))
			elif node.name == "title":
				if not node.hasProp("main"):
					self.gn.append(urllib.unquote(node.content))
				else:
					self.cn = urllib.unquote(node.content)
			elif node.name == "actor":
				self.actors.append(urllib.unquote(node.content))
			elif node.name == "director":
				self.director = urllib.unquote(node.content)
			elif node.name == "year":
				self.year = urllib.unquote(node.content)
			elif node.name == "country":
				self.country.append(urllib.unquote(node.content))
			elif node.name == "genre":
				self.genre.append(urllib.unquote(node.content))
			node = node.next
		node = xml.properties
		while node:
			if nodeSkip(node):
				pass
			elif node.name == "id":
				self.dn = urllib.unquote(node.content)
			node = node.next

class IMLDB:
	def __init__(self):
		self.base_dn = "ou=iml3,dc=void,dc=lz"
		self.ldap = ldap.open("localhost")
		self.bind_ro()
	def __del__(self):
		pass

	def bind_rw(self):
		return self.ldap.simple_bind_s("cn=admin,ou=iml3,dc=void,dc=lz", "iml3")

	def bind_ro(self):
		return self.ldap.simple_bind_s("", "")

	def checkdn(self, dn):
		if len(self.base_dn) > len(dn):
			raise RuntimeError("Base DN violation: %s" % dn)
		if dn[len(dn) - len(self.base_dn):] != self.base_dn:
			raise RuntimeError("Base DN violation: %s" % dn)
	def query(self, dn):
		self.checkdn(dn)
		films = self.ldap.search_s(dn, ldap.SCOPE_BASE, "objectClass=imlFilm")
		if len(films) < 1:
			return None
		film = IMLFilm(films[0], verbose=True)
		film.files(self.ldap.search_s(dn, ldap.SCOPE_SUBTREE, "objectClass=imlFile"))
		return film

	def search(self, search, mode = "title"):
		str = ldap.filter.escape_filter_chars(search)
		filter = "(| (cn=*%s*) (gn=*%s*))" % (str, str)
		if mode == "country":
			filter = "(co=*%s*)" % str
		elif mode == "genre":
			filter = "(imlFilmGenre=*%s*)" % str
		elif mode == "year":
			filter = "(imlFilmYear=%s)" % str
		films = self.ldap.search_s( self.base_dn, ldap.SCOPE_SUBTREE, 
			"(& (objectClass=imlFilm) %s )" % filter, attrlist=["cn"])
		return map (lambda x: IMLFilm(x), films)

	def person(self, search, mode = "director"):
		str = ldap.filter.escape_filter_chars(search)
		attr = "imlFilmDirector"
		if mode == "actor":
			attr = "imlFilmActor"
		films = self.ldap.search_s( self.base_dn, ldap.SCOPE_SUBTREE, 
			"(& (objectClass=imlFilm) (%s=*%s*))" % (attr, str), attrlist=["cn"])
		return map (lambda x: IMLFilm(x), films)

	def films(self):
		films = self.ldap.search_s(
			self.base_dn, ldap.SCOPE_SUBTREE, 
			"objectClass=imlFilm", attrlist=["cn"])
		return map (lambda x: IMLFilm(x), films)
	def file(self, dn, file):
		filedn = 'cn=%s,%s' % (ldap.dn.escape_dn_chars(file), dn)
		self.checkdn(filedn)
		try:
			files = self.ldap.search_s(filedn , ldap.SCOPE_BASE, "objectClass=imlFile")
			if files == []:
				return None
			return IMLFile(files[0])
		except:
			return None

	def modify_obj(self, film):
		self.checkdn(film.dn)
		attr = []
		if film.gn:
			attr.append((ldap.MOD_REPLACE, "givenName", film.gn))
		if film.actors:
			attr.append((ldap.MOD_REPLACE, "imlFilmActor", film.actors))
		if film.director:
			attr.append((ldap.MOD_REPLACE, "imlFilmDirector", film.director))
		if film.year:
			attr.append((ldap.MOD_REPLACE, "imlFilmYear", film.year))
		if film.genre:
			attr.append((ldap.MOD_REPLACE, "imlFilmGenre", film.genre))
		if film.country:
			attr.append((ldap.MOD_REPLACE, "co", film.country))
		self.ldap.modify_s(film.dn, attr)
		return film.dn
	def add_obj(self, film, recursive = True):
		try:
			filmdn = "cn=%s,%s" % (ldap.dn.escape_dn_chars(film.cn), self.base_dn)
			self.checkdn(filmdn)
			attr = [("objectClass", "imlFilm"), ("cn", film.cn)]
			if film.gn:
				attr.append(("givenName", film.gn))
			self.ldap.add_s(filmdn, attr)
			if recursive:
				map (lambda x: self.add_file_obj(filmdn, x), film.file)
			return filmdn
		except:
			return None
	def add_file_obj(self, film, file):
		self.checkdn(film)
		filedn = 'cn=%s,%s' % (ldap.dn.escape_dn_chars(file.cn), film)
		attr = [("objectClass", "imlFile"), ("cn", file.cn)]
		if file.file:
			attr.append(("imlFileURL", file.file))
		if file.lang:
			attr.append(("imlFileLang", file.lang))
			
		self.ldap.add_s(filedn, attr)
		return filedn
